package com.zilber.boot.activiti.service;

import com.zilber.boot.activiti.dto.TaskCompleteDTO;
import com.zilber.boot.activiti.dto.TaskQueryDTO;
import com.zilber.boot.activiti.vo.TaskVO;
import com.zilber.boot.exception.ServiceException;
import org.activiti.engine.task.Task;
import org.activiti.engine.task.TaskQuery;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;


import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service(value = "TaskService_")
public class TaskService {

    @Resource
    private org.activiti.engine.TaskService taskService;

    @Resource
    private InstanceService instanceService;

    public List<TaskVO> taskPage(TaskQueryDTO dto) {
        TaskQuery taskQuery = taskService.createTaskQuery()
                .active()
                .orderByTaskCreateTime()
                .desc();
        if (!StringUtils.isEmpty(dto.getAssignee())) {
            taskQuery.taskAssignee(dto.getAssignee());
        }
        if (!StringUtils.isEmpty(dto.getProcDefKey())) {
            taskQuery.processDefinitionKey(dto.getProcDefKey());
        }
        if (!StringUtils.isEmpty(dto.getProcInstId())) {
            taskQuery.processInstanceId(dto.getProcInstId());
        }
        List<Task> tasks = taskQuery.listPage(dto.getPageNo() * dto.getPageSize(), dto.getPageSize());
        return TaskVO.merge(tasks);
    }

    public void complete(String userCode, String taskId, TaskCompleteDTO dto) {
        Task task = taskService.createTaskQuery()
                .taskAssignee(userCode)
                .taskId(taskId)
                .singleResult();

        if (task == null) {
            throw new ServiceException("没有任务");
        }

        if (task.getAssignee() == null) {
            taskService.claim(taskId, userCode);
        }
        if (!StringUtils.isEmpty(dto.getRemark())) {
            //taskService.createComment(taskId, task.getProcessInstanceId(), dto.getRemark());
            taskService.addComment(taskId, task.getProcessInstanceId(), dto.getRemark());
        }
        Map<String, Object> variables = dto.getVariables();
        if (dto.getVariables() == null) {
            variables = new HashMap<>();
        }
        if (dto.getStatus() == 0) {
            // TODO: 多实例并行任务，需要按照业务处理，是否一个人审批不通过，就都不通过
            taskService.setVariable(taskId, "status", 0);
            instanceService.failEndById(task.getProcessInstanceId(), dto.getRemark());
        } else {
            variables.put("status", dto.getStatus());
            taskService.complete(taskId, variables);
        }
    }

    /*public List<io.izn.iec.camunda.vo.SimpleTaskVO> getSimpleTasks(List<Integer> businessIds, DefinitionKey definitionKey) {
        String sql = "SELECT\n" +
                "       t2.ID_ as taskId,\n" +
                "       t2.NAME_ as taskName,\n" +
                "       t2.DESCRIPTION_ as taskDescription,\n" +
                "       t2.TASK_DEF_KEY_ as taskDefinitionKey,\n" +
                "       t2.ASSIGNEE_ as assignee,\n" +
                "       t2.CREATE_TIME_ as taskTime,\n" +
                "       t2.PRIORITY_ as priority,\n" +
                "       t2.SUSPENSION_STATE_ as suspensionState,\n" +
                "       t1.business_id as businessId\n" +
                "FROM bpm_business as t1\n" +
                "         LEFT JOIN ACT_RU_TASK as t2 on t1.proc_inst_id = t2.PROC_INST_ID_\n" +
                "WHERE t1.business_id in (?1) and t1.proc_def_key = ?2";

        Query nativeQuery = entityManager.createNativeQuery(sql);
        nativeQuery.setParameter(1, businessIds);
        nativeQuery.setParameter(2, definitionKey.getValue());
        return nativeQuery
                .unwrap(NativeQueryImpl.class)
                .setResultTransformer(Transformers.aliasToBean(io.izn.iec.camunda.vo.SimpleTaskVO.class))
                .list();
    }

    public List<io.izn.iec.camunda.vo.SimpleTaskVO> getSimpleTasks(List<Integer> businessIds, List<DefinitionKey> definitionKey) {
        List<String> collect = definitionKey.stream().map(DefinitionKey::getValue).collect(Collectors.toList());
        String sql = "SELECT\n" +
                "       t2.ID_ as taskId,\n" +
                "       t2.NAME_ as taskName,\n" +
                "       t2.DESCRIPTION_ as taskDescription,\n" +
                "       t2.TASK_DEF_KEY_ as taskDefinitionKey,\n" +
                "       t2.ASSIGNEE_ as assignee,\n" +
                "       t2.CREATE_TIME_ as taskTime,\n" +
                "       t2.PRIORITY_ as priority,\n" +
                "       t2.SUSPENSION_STATE_ as suspensionState,\n" +
                "       t1.business_id as businessId\n" +
                "FROM bpm_business as t1\n" +
                "         LEFT JOIN ACT_RU_TASK as t2 on t1.proc_inst_id = t2.PROC_INST_ID_\n" +
                "WHERE t1.business_id in (?1) and t1.proc_def_key in (?2)";

        Query nativeQuery = entityManager.createNativeQuery(sql);
        nativeQuery.setParameter(1, businessIds);
        nativeQuery.setParameter(2, collect);
        return nativeQuery
                .unwrap(NativeQueryImpl.class)
                .setResultTransformer(Transformers.aliasToBean(io.izn.iec.camunda.vo.SimpleTaskVO.class))
                .list();
    }*/
}
